<?php

namespace FiloBlu\Job\Controller\Send;

use Exception;
use FiloBlu\Job\Model\Mail\TransportBuilder;
use Magento\Framework\App\Action\Action;
use Magento\Framework\App\Action\Context;
use Magento\Framework\App\Area;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\App\Filesystem\DirectoryList;
use Magento\Framework\App\Request\DataPersistorInterface;
use Magento\Framework\DataObject;
use Magento\Framework\Exception\FileSystemException;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\Exception\MailException;
use Magento\Framework\Filesystem;
use Magento\Framework\Filesystem\Driver\File;
use Magento\Framework\Phrase;
use Magento\Framework\Translate\Inline\StateInterface;
use Magento\MediaStorage\Model\File\UploaderFactory;
use Magento\Store\Model\ScopeInterface;
use Magento\Store\Model\Store;
use Psr\Log\LoggerInterface;
use Throwable;
use Zend_Validate;
use Zend_Validate_Exception;

/**
 * Class Request
 * @package FiloBlu\Job\Controller\Send
 */
class Request extends Action
{
    /**
     * Recipient email config path
     */
    const XML_PATH_EMAIL_RECIPIENT = 'filoblu_job_section/email/recipient_email';
    /**
     * Sender email config path
     */
    const XML_PATH_EMAIL_SENDER = 'filoblu_job_section/email/sender_email_identity';
    /**
     * Email template config path
     */
    const XML_PATH_EMAIL_TEMPLATE = 'filoblu_job_section/email/email_template';
    /**
     * @var array
     */
    protected $allowedExtensions = ['docx', 'doc', 'pdf', 'odt'];
    /**
     * @var string
     */
    protected $fileId = 'attachment';
    /**
     * @var int
     */
    protected $maxFileSize = 1500000;
    /**
     * @var DataPersistorInterface
     */
    private $dataPersistor;
    /**
     * @var Filesystem
     */
    protected $fileSystem;
    /**
     * @var LoggerInterface
     */
    protected $logger;
    /**
     * @var UploaderFactory
     */
    protected $uploaderFactory;
    /**
     * @var TransportBuilder
     */
    protected $transportBuilder;
    /**
     * @var ScopeConfigInterface
     */
    protected $scopeConfig;
    /**
     * @var File
     */
    protected $file;
    /**
     * @var StateInterface
     */
    protected $inlineTranslation;

    /**
     * Request constructor.
     * @param Context $context
     * @param TransportBuilder $transportBuilder
     * @param StateInterface $inlineTranslation
     * @param ScopeConfigInterface $scopeConfig
     * @param Filesystem $fileSystem
     * @param UploaderFactory $uploaderFactory
     * @param File $file
     * @param LoggerInterface $logger
     * @param DataPersistorInterface $dataPersistor
     */
    public function __construct(
        Context $context,
        TransportBuilder $transportBuilder,
        StateInterface $inlineTranslation,
        ScopeConfigInterface $scopeConfig,
        Filesystem $fileSystem,
        UploaderFactory $uploaderFactory,
        File $file,
        LoggerInterface $logger,
        DataPersistorInterface $dataPersistor
    ) {
        parent::__construct($context);
        $this->transportBuilder = $transportBuilder;
        $this->inlineTranslation = $inlineTranslation;
        $this->scopeConfig = $scopeConfig;
        $this->fileSystem = $fileSystem;
        $this->uploaderFactory = $uploaderFactory;
        $this->file = $file;
        $this->logger = $logger;
        $this->dataPersistor = $dataPersistor;
    }

    /**
     * Post user question
     *
     * @return void
     * @throws Exception
     */
    public function execute()
    {
        $post = $this->getRequest()->getPostValue();

        if (!$post) {
            $this->_redirect($this->_redirect->getRefererUrl());
            return;
        }

        try {
            $this->validateRequest()->sendEmail();
        } catch (Exception $exception) {
            $this->messageManager->addErrorMessage($exception->getMessage());
            $this->logger->error($exception->getMessage(), ['exception' => $exception]);
        } catch (Throwable $throwable) {
            $this->messageManager->addErrorMessage($throwable->getMessage());
            $this->logger->error($throwable->getMessage(), ['exception' => $throwable]);
        } finally {
            $this->_redirect($this->_redirect->getRefererUrl());
        }
    }

    /**
     * @return Request
     * @throws LocalizedException
     * @throws Zend_Validate_Exception
     */
    public function validateRequest()
    {
        $post = $this->getRequest()->getPostValue();
        $postFile = $this->getRequest()->getFiles();

        $errors = [];

        if (!Zend_Validate::is(trim($post['name']), 'NotEmpty')) {
            $errors[] = __('Name field must be not empty.');
        }

        if (!Zend_Validate::is(trim($post['email']), 'EmailAddress')) {
            $errors[] = __('Insert a valid email address.');
        }

        if (!isset($postFile['attachment']['name']) || empty($postFile['attachment']['name'])) {
            $errors[] = __('Attachment needed.');
        }

        if ($postFile['attachment']['size'] > $this->maxFileSize) {
            $errors[] = __('Exceeded file size.');
        }

        $info = explode('.', $postFile['attachment']['name']);
        $extension = strtolower(end($info));

        if (!in_array($extension, $this->allowedExtensions, true)) {
            $errors[] = __('File extension not allowed.');
        }

        if (empty($errors)) {
            return $this;
        }

        throw new LocalizedException(new Phrase(implode(PHP_EOL, $errors)));
    }

    /**
     * @throws FileSystemException
     * @throws LocalizedException
     * @throws MailException
     * @throws Exception
     */
    protected function sendEmail()
    {
        $postFile = $this->getRequest()->getFiles();
        $post = $this->getRequest()->getPostValue();

        if (!isset($post['interestArea'])) {
            $post['interestArea'] = '';
        }

        if (!isset($post['country'])) {
            $post['country'] = '';
        }

        $postObject = new DataObject();
        $postObject->setData($post);

        $fileName = preg_replace('/[^a-z0-9_\\-.]+/i', '_', $postFile['attachment']['name']);

        $destinationPath = $this->getDestinationPath();

        $uploader = $this->uploaderFactory->create(['fileId' => $this->fileId])
            ->setAllowCreateFolders(true)
            ->setAllowedExtensions($this->allowedExtensions);

        $uploader->save($destinationPath);

        $this->transportBuilder
            ->setTemplateIdentifier($this->scopeConfig->getValue(self::XML_PATH_EMAIL_TEMPLATE, ScopeInterface::SCOPE_STORE))
            ->setTemplateOptions(
                [
                    'area' => Area::AREA_FRONTEND,
                    'store' => Store::DEFAULT_STORE_ID,
                ]
            )
            ->setTemplateVars(['data' => $postObject])
            ->setFromByScope($this->scopeConfig->getValue(self::XML_PATH_EMAIL_SENDER))
            ->addTo($this->scopeConfig->getValue(self::XML_PATH_EMAIL_RECIPIENT))
            ->setReplyTo($post['email']);

        if (isset($postFile['attachment']['name']) && !empty($postFile['attachment']['name'])) {
            $this->transportBuilder->addAttachment($destinationPath . $fileName, $fileName);
        }

        $transport = $this->transportBuilder->getTransport();
        $transport->sendMessage();

        if ($this->file->isExists($destinationPath . $fileName)) {
            $this->file->deleteFile($destinationPath . $fileName);
        }

        $this->inlineTranslation->resume();
        $this->messageManager->addSuccessMessage(__('Thanks for contacting us. We\'ll respond to you very soon.'));

        $this->dataPersistor->clear('job');
    }

    /**
     * @return string
     * @throws FileSystemException
     */
    public function getDestinationPath()
    {
        return $this->fileSystem
            ->getDirectoryWrite(DirectoryList::TMP)
            ->getAbsolutePath('/');
    }
}
